import { InviteUserService } from '../services/invite-user.service';
import {
  Body,
  Controller,
  Delete,
  Param,
  Post,
  Put,
  Req,
  UseGuards,
} from '@nestjs/common';
import { ApiTags, ApiBearerAuth, ApiOperation } from '@nestjs/swagger';
import { AddUserInvitationDto } from '../dtos/add.invite-user.dto';
import { ControllerRoute } from '@app/common/constants/controller-route';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { PermissionsGuard } from 'src/guards/permissions.guard';
import { Permissions } from 'src/decorators/permissions.decorator';
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import { CheckEmailDto } from '../dtos/check-email.dto';
import { ActivateCodeDto } from '../dtos/active-code.dto';
import {
  DisableUserInvitationDto,
  UpdateUserInvitationDto,
} from '../dtos/update.invite-user.dto';

@ApiTags('Invite User Module')
@Controller({
  version: '1',
  path: ControllerRoute.INVITE_USER.ROUTE,
})
export class InviteUserController {
  constructor(private readonly inviteUserService: InviteUserService) {}

  @ApiBearerAuth()
  @UseGuards(PermissionsGuard)
  @Permissions('USER_ADD')
  @Post()
  @ApiOperation({
    summary: ControllerRoute.INVITE_USER.ACTIONS.CREATE_USER_INVITATION_SUMMARY,
    description:
      ControllerRoute.INVITE_USER.ACTIONS.CREATE_USER_INVITATION_DESCRIPTION,
  })
  async createUserInvitation(
    @Body() addUserInvitationDto: AddUserInvitationDto,
    @Req() request: any,
  ): Promise<BaseResponseDto> {
    const user = request.user;
    return await this.inviteUserService.createUserInvitation(
      addUserInvitationDto,
      user.role.type,
    );
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post('check-email')
  @ApiOperation({
    summary: ControllerRoute.INVITE_USER.ACTIONS.CHECK_EMAIL_SUMMARY,
    description: ControllerRoute.INVITE_USER.ACTIONS.CHECK_EMAIL_DESCRIPTION,
  })
  async checkEmailAndProject(
    @Body() addUserInvitationDto: CheckEmailDto,
  ): Promise<BaseResponseDto> {
    return await this.inviteUserService.checkEmailAndProject(
      addUserInvitationDto,
    );
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post('activation')
  @ApiOperation({
    summary: ControllerRoute.INVITE_USER.ACTIONS.ACTIVATION_CODE_SUMMARY,
    description:
      ControllerRoute.INVITE_USER.ACTIONS.ACTIVATION_CODE_DESCRIPTION,
  })
  async activationCodeController(
    @Body() activateCodeDto: ActivateCodeDto,
  ): Promise<BaseResponseDto> {
    return await this.inviteUserService.activationCode(activateCodeDto);
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put(':invitedUserUuid')
  @ApiOperation({
    summary: ControllerRoute.INVITE_USER.ACTIONS.UPDATE_USER_INVITATION_SUMMARY,
    description:
      ControllerRoute.INVITE_USER.ACTIONS.UPDATE_USER_INVITATION_DESCRIPTION,
  })
  async updateUserInvitation(
    @Param('invitedUserUuid') invitedUserUuid: string,
    @Body() updateUserInvitationDto: UpdateUserInvitationDto,
  ): Promise<BaseResponseDto> {
    return await this.inviteUserService.updateUserInvitation(
      updateUserInvitationDto,
      invitedUserUuid,
    );
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put(':invitedUserUuid/disable')
  @ApiOperation({
    summary:
      ControllerRoute.INVITE_USER.ACTIONS.DISABLE_USER_INVITATION_SUMMARY,
    description:
      ControllerRoute.INVITE_USER.ACTIONS.DISABLE_USER_INVITATION_DESCRIPTION,
  })
  async disableUserInvitation(
    @Param('invitedUserUuid') invitedUserUuid: string,
    @Body() disableUserInvitationDto: DisableUserInvitationDto,
  ): Promise<BaseResponseDto> {
    return await this.inviteUserService.disableUserInvitation(
      disableUserInvitationDto,
      invitedUserUuid,
    );
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Delete(':invitedUserUuid')
  @ApiOperation({
    summary: ControllerRoute.INVITE_USER.ACTIONS.DELETE_USER_INVITATION_SUMMARY,
    description:
      ControllerRoute.INVITE_USER.ACTIONS.DELETE_USER_INVITATION_DESCRIPTION,
  })
  async deleteUserInvitation(
    @Param('invitedUserUuid') invitedUserUuid: string,
  ): Promise<BaseResponseDto> {
    return await this.inviteUserService.deleteUserInvitation(invitedUserUuid);
  }
}
