import { VisitorPasswordRepository } from './../../../libs/common/src/modules/visitor-password/repositories/visitor-password.repository';
import {
  Injectable,
  HttpException,
  HttpStatus,
  BadRequestException,
} from '@nestjs/common';
import { TuyaContext } from '@tuya/tuya-connector-nodejs';
import { ConfigService } from '@nestjs/config';
import {
  addDeviceObjectInterface,
  createTickInterface,
} from '../interfaces/visitor-password.interface';
import { DeviceRepository } from '@app/common/modules/device/repositories';
import { ProductType } from '@app/common/constants/product-type.enum';

import { AddDoorLockTemporaryPasswordDto } from '../dtos';
import { EmailService } from '@app/common/util/email.service';
import { PasswordEncryptionService } from 'src/door-lock/services/encryption.services';
import { DoorLockService } from 'src/door-lock/services';
import { DeviceService } from 'src/device/services';
import { DeviceStatuses } from '@app/common/constants/device-status.enum';
import {
  DaysEnum,
  EnableDisableStatusEnum,
} from '@app/common/constants/days.enum';
import { PasswordType } from '@app/common/constants/password-type.enum';
import {
  CommonHourMinutes,
  CommonHours,
} from '@app/common/constants/hours-minutes.enum';
import { ProjectRepository } from '@app/common/modules/project/repositiories';
import { VisitorPasswordEnum } from '@app/common/constants/visitor-password.enum';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';
import { Not } from 'typeorm';
import { ORPHAN_SPACE_NAME } from '@app/common/constants/orphan-constant';

@Injectable()
export class VisitorPasswordService {
  private tuya: TuyaContext;
  constructor(
    private readonly configService: ConfigService,
    private readonly deviceRepository: DeviceRepository,
    private readonly visitorPasswordRepository: VisitorPasswordRepository,
    private readonly emailService: EmailService,
    private readonly doorLockService: DoorLockService,
    private readonly deviceService: DeviceService,
    private readonly passwordEncryptionService: PasswordEncryptionService,
    private readonly projectRepository: ProjectRepository,
  ) {
    const accessKey = this.configService.get<string>('auth-config.ACCESS_KEY');
    const secretKey = this.configService.get<string>('auth-config.SECRET_KEY');
    const tuyaEuUrl = this.configService.get<string>('tuya-config.TUYA_EU_URL');
    this.tuya = new TuyaContext({
      baseUrl: tuyaEuUrl,
      accessKey,
      secretKey,
    });
  }
  async handleTemporaryPassword(
    addDoorLockTemporaryPasswordDto: AddDoorLockTemporaryPasswordDto,
    userUuid: string,
    projectUuid: string,
  ) {
    const { operationType } = addDoorLockTemporaryPasswordDto;

    let result;

    switch (operationType) {
      case VisitorPasswordEnum.ONLINE_ONE_TIME:
        result = await this.addOnlineTemporaryPasswordOneTime(
          addDoorLockTemporaryPasswordDto,
          userUuid,
          projectUuid,
        );
        break;
      case VisitorPasswordEnum.ONLINE_MULTIPLE_TIME:
        result = await this.addOnlineTemporaryPasswordMultipleTime(
          addDoorLockTemporaryPasswordDto,
          userUuid,
          projectUuid,
        );
        break;
      case VisitorPasswordEnum.OFFLINE_ONE_TIME:
        result = await this.addOfflineOneTimeTemporaryPassword(
          addDoorLockTemporaryPasswordDto,
          userUuid,
          projectUuid,
        );
        break;
      case VisitorPasswordEnum.OFFLINE_MULTIPLE_TIME:
        result = await this.addOfflineMultipleTimeTemporaryPassword(
          addDoorLockTemporaryPasswordDto,
          userUuid,
          projectUuid,
        );
        break;
      default:
        throw new BadRequestException('Invalid operation type');
    }

    return new SuccessResponseDto({
      message: `Successfully created new ${operationType.toLowerCase()} visitor password`,
      data: result,
      statusCode: HttpStatus.CREATED,
    });
  }
  async addOfflineMultipleTimeTemporaryPassword(
    addDoorLockOfflineMultipleDto: AddDoorLockTemporaryPasswordDto,
    userUuid: string,
    projectUuid: string,
  ) {
    try {
      const deviceResults = await Promise.allSettled(
        addDoorLockOfflineMultipleDto.devicesUuid.map(async (deviceUuid) => {
          try {
            const deviceDetails = await this.getDeviceByDeviceUuid(
              deviceUuid,
              true,
              projectUuid,
            );

            if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
              throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
            }

            if (deviceDetails.productDevice.prodType !== ProductType.DL) {
              throw new HttpException(
                'This is not a door lock device',
                HttpStatus.BAD_REQUEST,
              );
            }

            const createMultipleOfflinePass =
              await this.addOfflineTemporaryPasswordTuya(
                deviceDetails.deviceTuyaUuid,
                PasswordType.MULTIPLE,
                addDoorLockOfflineMultipleDto,
                addDoorLockOfflineMultipleDto.passwordName,
              );

            if (!createMultipleOfflinePass.success) {
              throw new HttpException(
                createMultipleOfflinePass.msg,
                HttpStatus.BAD_REQUEST,
              );
            }

            // Send email if the password creation is successful
            if (createMultipleOfflinePass.result.offline_temp_password_id) {
              const emailSubject = 'One-Time Password Creation Success';
              const emailBody = `Your multiple time offline temporary password "${createMultipleOfflinePass.result.offline_temp_password}" has been successfully created for the device with UUID: ${deviceUuid}.\n\nThank you for using our service.\n\nBest Regards,\nSyncrow`;

              await this.emailService.sendEmail(
                addDoorLockOfflineMultipleDto.email,
                emailSubject,
                emailBody,
              );
            }
            if (createMultipleOfflinePass.result.offline_temp_password_id) {
              await this.visitorPasswordRepository.save({
                passwordTuyaUuid: `${createMultipleOfflinePass.result.offline_temp_password_id}`,
                user: {
                  uuid: userUuid,
                },
              });
            }
            return {
              success: true,
              result: createMultipleOfflinePass.result,
              deviceUuid,
            };
          } catch (error) {
            return {
              success: false,
              deviceUuid,
              error: error.message || 'Error processing device',
            };
          }
        }),
      );

      // Process results: separate successful and failed operations
      const successfulResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.FULLFILLED &&
            (result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) => (result as PromiseFulfilledResult<any>).value);

      const failedResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.REJECTED ||
            !(result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) =>
          result.status === DeviceStatuses.REJECTED
            ? {
                success: false,
                error:
                  (result as PromiseRejectedResult).reason.message ||
                  'Error processing device',
              }
            : (result as PromiseFulfilledResult<any>).value,
        );

      // Return results if there are successful operations
      if (successfulResults.length > 0) {
        return {
          successOperations: successfulResults,
          failedOperations: failedResults,
        };
      }

      // If all failed, throw an error with all failure messages
      throw new HttpException(
        failedResults.map((res) => res.error).join('; ') ||
          'All device operations failed',
        HttpStatus.BAD_REQUEST,
      );
    } catch (error) {
      console.error(error);

      throw new HttpException(
        error.message ||
          'Error adding offline multiple-time temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async addOfflineOneTimeTemporaryPassword(
    addDoorLockOfflineOneTimeDto: AddDoorLockTemporaryPasswordDto,
    userUuid: string,
    projectUuid: string,
  ) {
    try {
      const deviceResults = await Promise.allSettled(
        addDoorLockOfflineOneTimeDto.devicesUuid.map(async (deviceUuid) => {
          try {
            const deviceDetails = await this.getDeviceByDeviceUuid(
              deviceUuid,
              true,
              projectUuid,
            );

            if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
              throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
            }

            if (deviceDetails.productDevice.prodType !== ProductType.DL) {
              throw new HttpException(
                'This is not a door lock device',
                HttpStatus.BAD_REQUEST,
              );
            }

            const createOnceOfflinePass =
              await this.addOfflineTemporaryPasswordTuya(
                deviceDetails.deviceTuyaUuid,
                'once',
                null,
                addDoorLockOfflineOneTimeDto.passwordName,
              );

            if (!createOnceOfflinePass.success) {
              throw new HttpException(
                createOnceOfflinePass.msg,
                HttpStatus.BAD_REQUEST,
              );
            }

            // Send email if the password creation is successful
            if (createOnceOfflinePass.result.offline_temp_password_id) {
              const emailSubject = 'One-Time Password Creation Success';
              const emailBody = `Your one-time offline temporary password "${createOnceOfflinePass.result.offline_temp_password}" has been successfully created for the device with UUID: ${deviceUuid}.\n\nThank you for using our service.\n\nBest Regards,\nSyncrow`;

              await this.emailService.sendEmail(
                addDoorLockOfflineOneTimeDto.email,
                emailSubject,
                emailBody,
              );
            }
            if (createOnceOfflinePass.result.offline_temp_password_id) {
              await this.visitorPasswordRepository.save({
                passwordTuyaUuid: `${createOnceOfflinePass.result.offline_temp_password_id}`,
                user: {
                  uuid: userUuid,
                },
              });
            }
            return {
              success: true,
              result: createOnceOfflinePass.result,
              deviceUuid,
            };
          } catch (error) {
            return {
              success: false,
              deviceUuid,
              error: error.message || 'Error processing device',
            };
          }
        }),
      );

      // Process results: separate successful and failed operations
      const successfulResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.FULLFILLED &&
            (result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) => (result as PromiseFulfilledResult<any>).value);

      const failedResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.REJECTED ||
            !(result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) =>
          result.status === DeviceStatuses.REJECTED
            ? {
                success: false,
                error:
                  (result as PromiseRejectedResult).reason.message ||
                  'Error processing device',
              }
            : (result as PromiseFulfilledResult<any>).value,
        );

      // Return results if there are successful operations
      if (successfulResults.length > 0) {
        return {
          successOperations: successfulResults,
          failedOperations: failedResults,
        };
      }

      // If all failed, throw an error with all failure messages
      throw new HttpException(
        failedResults.map((res) => res.error).join('; ') ||
          'All device operations failed',
        HttpStatus.BAD_REQUEST,
      );
    } catch (error) {
      console.error(error);

      throw new HttpException(
        error.message ||
          'Error adding offline one-time temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async addOfflineTemporaryPasswordTuya(
    doorLockUuid: string,
    type: string,
    addDoorLockOfflineMultipleDto: AddDoorLockTemporaryPasswordDto,
    passwordName: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.1/devices/${doorLockUuid}/door-lock/offline-temp-password`;

      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          ...(type === PasswordType.MULTIPLE && {
            effective_time: addDoorLockOfflineMultipleDto.effectiveTime,
            invalid_time: addDoorLockOfflineMultipleDto.invalidTime,
          }),
          name: passwordName,
          type,
        },
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        'Error adding offline temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async addOnlineTemporaryPasswordMultipleTime(
    addDoorLockOnlineMultipleDto: AddDoorLockTemporaryPasswordDto,
    userUuid: string,
    projectUuid: string,
  ) {
    try {
      const deviceResults = await Promise.allSettled(
        addDoorLockOnlineMultipleDto.devicesUuid.map(async (deviceUuid) => {
          try {
            const passwordData = await this.getTicketAndEncryptedPassword(
              deviceUuid,
              addDoorLockOnlineMultipleDto.password,
              projectUuid,
            );

            if (
              !passwordData.ticketKey ||
              !passwordData.encryptedPassword ||
              !passwordData.deviceTuyaUuid
            ) {
              throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
            }

            const addDeviceObj: addDeviceObjectInterface = {
              ...addDoorLockOnlineMultipleDto,
              ...passwordData,
            };

            const createPass =
              await this.addOnlineTemporaryPasswordMultipleTuya(
                addDeviceObj,
                passwordData.deviceTuyaUuid,
              );

            if (!createPass.success) {
              throw new HttpException(createPass.msg, HttpStatus.BAD_REQUEST);
            }

            // Send email if the password creation is successful
            if (createPass.result.id) {
              const emailSubject = 'Password Creation Success';
              const emailBody = `Your temporary password "${addDoorLockOnlineMultipleDto.password}" has been successfully created for the device with UUID: ${deviceUuid}.\n\nThank you for using our service.\n\nBest Regards,\nSyncrow`;

              await this.emailService.sendEmail(
                addDoorLockOnlineMultipleDto.email,
                emailSubject,
                emailBody,
              );
            }
            if (createPass.result.id) {
              await this.visitorPasswordRepository.save({
                passwordTuyaUuid: `${createPass.result.id}`,
                user: {
                  uuid: userUuid,
                },
              });
            }
            return {
              success: true,
              id: createPass.result.id,
              deviceUuid,
            };
          } catch (error) {
            return {
              success: false,
              deviceUuid,
              error: error.message || 'Error processing device',
            };
          }
        }),
      );

      // Process results: separate successful and failed operations
      const successfulResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.FULLFILLED &&
            (result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) => (result as PromiseFulfilledResult<any>).value);

      const failedResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.REJECTED ||
            !(result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) =>
          result.status === DeviceStatuses.REJECTED
            ? {
                success: false,
                error:
                  (result as PromiseRejectedResult).reason.message ||
                  'Error processing device',
              }
            : (result as PromiseFulfilledResult<any>).value,
        );

      // Return results if there are successful operations
      if (successfulResults?.length > 0) {
        return {
          successOperations: successfulResults,
          failedOperations: failedResults,
        };
      }

      // Throw an error if all operations failed
      throw new HttpException(
        failedResults.map((res) => res.error).join('; ') ||
          'All device operations failed',
        HttpStatus.BAD_REQUEST,
      );
    } catch (error) {
      console.error(error);

      throw new HttpException(
        error.message || 'Error adding online temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getPasswords(projectUuid: string) {
    await this.validateProject(projectUuid);

    const deviceIds = await this.deviceRepository.find({
      where: {
        productDevice: {
          prodType: ProductType.DL,
        },
        spaceDevice: {
          spaceName: Not(ORPHAN_SPACE_NAME),
          community: {
            project: {
              uuid: projectUuid,
            },
          },
        },
        isActive: true,
      },
    });

    const data = [];
    deviceIds.forEach((deviceId) => {
      data.push(
        this.doorLockService
          .getOnlineTemporaryPasswordsOneTime(deviceId.uuid, true, false)
          .catch(() => {}),
        this.doorLockService
          .getOnlineTemporaryPasswordsOneTime(deviceId.uuid, true, true)
          .catch(() => {}),
        this.doorLockService
          .getOnlineTemporaryPasswordsMultiple(deviceId.uuid, true, false)
          .catch(() => {}),
        this.doorLockService
          .getOnlineTemporaryPasswordsMultiple(deviceId.uuid, true, true)
          .catch(() => {}),
        this.doorLockService
          .getOfflineOneTimeTemporaryPasswords(deviceId.uuid, true, false)
          .catch(() => {}),
        this.doorLockService
          .getOfflineOneTimeTemporaryPasswords(deviceId.uuid, true, true)
          .catch(() => {}),
        this.doorLockService
          .getOfflineMultipleTimeTemporaryPasswords(deviceId.uuid, true, false)
          .catch(() => {}),
        this.doorLockService
          .getOfflineMultipleTimeTemporaryPasswords(deviceId.uuid, true, true)
          .catch(() => {}),
      );
    });
    const result = (await Promise.all(data)).flat().filter((datum) => {
      return datum != null;
    });

    return new SuccessResponseDto({
      message: 'Successfully retrieved temporary passwords',
      data: result,
      statusCode: HttpStatus.OK,
    });
  }

  async addOnlineTemporaryPasswordOneTime(
    addDoorLockOnlineOneTimeDto: AddDoorLockTemporaryPasswordDto,
    userUuid: string,
    projectUuid: string,
  ) {
    try {
      const deviceResults = await Promise.allSettled(
        addDoorLockOnlineOneTimeDto.devicesUuid.map(async (deviceUuid) => {
          try {
            const passwordData = await this.getTicketAndEncryptedPassword(
              deviceUuid,
              addDoorLockOnlineOneTimeDto.password,
              projectUuid,
            );

            if (
              !passwordData.ticketKey ||
              !passwordData.encryptedPassword ||
              !passwordData.deviceTuyaUuid
            ) {
              throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
            }

            const addDeviceObj: addDeviceObjectInterface = {
              ...addDoorLockOnlineOneTimeDto,
              ...passwordData,
            };

            const createPass = await this.addOnlineTemporaryPasswordOneTimeTuya(
              addDeviceObj,
              passwordData.deviceTuyaUuid,
            );

            if (!createPass.success) {
              throw new HttpException(createPass.msg, HttpStatus.BAD_REQUEST);
            }

            // Send email if the password creation is successful
            if (createPass.result.id) {
              const emailSubject = 'Password Creation Success';
              const emailBody = `Your temporary password "${addDoorLockOnlineOneTimeDto.password}" has been successfully created for the device with UUID: ${deviceUuid}.\n\nThank you for using our service.\n\nBest Regards,\nSyncrow`;

              await this.emailService.sendEmail(
                addDoorLockOnlineOneTimeDto.email,
                emailSubject,
                emailBody,
              );
            }
            if (createPass.result.id) {
              await this.visitorPasswordRepository.save({
                passwordTuyaUuid: `${createPass.result.id}`,
                user: {
                  uuid: userUuid,
                },
              });
            }
            return {
              success: true,
              id: createPass.result.id,
              deviceUuid,
            };
          } catch (error) {
            return {
              success: false,
              deviceUuid,
              error: error.message || 'Error processing device',
            };
          }
        }),
      );

      // Process results: separate successful and failed operations
      const successfulResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.FULLFILLED &&
            (result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) => (result as PromiseFulfilledResult<any>).value);

      const failedResults = deviceResults
        .filter(
          (result) =>
            result.status === DeviceStatuses.REJECTED ||
            !(result as PromiseFulfilledResult<any>).value.success,
        )
        .map((result) =>
          result.status === DeviceStatuses.REJECTED
            ? {
                success: false,
                error:
                  (result as PromiseRejectedResult).reason.message ||
                  'Error processing device',
              }
            : (result as PromiseFulfilledResult<any>).value,
        );

      // Return results if there are successful operations
      if (successfulResults.length > 0) {
        return {
          successOperations: successfulResults,
          failedOperations: failedResults,
        };
      }

      // Throw an error if all operations failed
      throw new HttpException(
        failedResults.map((res) => res.error).join('; ') ||
          'All device operations failed',
        HttpStatus.BAD_REQUEST,
      );
    } catch (error) {
      console.error(error);

      throw new HttpException(
        error.message || 'Error adding online temporary password from Tuya',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getTicketAndEncryptedPassword(
    doorLockUuid: string,
    passwordPlan: string,
    projectUuid: string,
  ) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(
        doorLockUuid,
        true,
        projectUuid,
      );

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      } else if (deviceDetails.productDevice.prodType !== ProductType.DL) {
        throw new HttpException(
          'This is not a door lock device',
          HttpStatus.BAD_REQUEST,
        );
      }
      const ticketDetails = await this.createDoorLockTicketTuya(
        deviceDetails.deviceTuyaUuid,
      );

      if (!ticketDetails.result.ticket_id || !ticketDetails.result.ticket_key) {
        throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
      }
      const decrypted =
        this.passwordEncryptionService.generateEncryptedPassword(
          passwordPlan,
          ticketDetails.result.ticket_key,
        );
      return {
        ticketId: ticketDetails.result.ticket_id,
        ticketKey: ticketDetails.result.ticket_key,
        encryptedPassword: decrypted,
        deviceTuyaUuid: deviceDetails.deviceTuyaUuid,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Error processing the request',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async createDoorLockTicketTuya(
    deviceUuid: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.0/smart-lock/devices/${deviceUuid}/password-ticket`;
      const response = await this.tuya.request({
        method: 'POST',
        path,
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        'Error creating door lock ticket from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async addOnlineTemporaryPasswordMultipleTuya(
    addDeviceObj: addDeviceObjectInterface,
    doorLockUuid: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.0/devices/${doorLockUuid}/door-lock/temp-password`;
      let scheduleList;
      if (addDeviceObj?.scheduleList?.length > 0) {
        scheduleList = addDeviceObj.scheduleList.map((schedule) => ({
          effective_time: this.timeToMinutes(schedule.effectiveTime),
          invalid_time: this.timeToMinutes(schedule.invalidTime),
          working_day: this.getWorkingDayValue(schedule.workingDay),
        }));
      }

      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          name: addDeviceObj.passwordName,
          password: addDeviceObj.encryptedPassword,
          effective_time: addDeviceObj.effectiveTime,
          invalid_time: addDeviceObj.invalidTime,
          password_type: 'ticket',
          ticket_id: addDeviceObj.ticketId,
          ...(addDeviceObj?.scheduleList?.length > 0 && {
            schedule_list: scheduleList,
          }),

          type: EnableDisableStatusEnum.DISABLED,
        },
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        error.msg || 'Error adding online temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  getWorkingDayValue(days) {
    // Array representing the days of the week
    const weekDays = [
      DaysEnum.SAT,
      DaysEnum.FRI,
      DaysEnum.THU,
      DaysEnum.WED,
      DaysEnum.TUE,
      DaysEnum.MON,
      DaysEnum.SUN,
    ];

    // Initialize a binary string with 7 bits
    let binaryString = '0000000';

    // Iterate through the input array and update the binary string
    days.forEach((day) => {
      const index = weekDays.indexOf(day);
      if (index !== -1) {
        // Set the corresponding bit to EnableDisableStatusEnum.ENABLED
        binaryString =
          binaryString.substring(0, index) +
          EnableDisableStatusEnum.ENABLED +
          binaryString.substring(index + 1);
      }
    });

    // Convert the binary string to an integer
    const workingDayValue = parseInt(binaryString, 2);

    return workingDayValue;
  }
  getDaysFromWorkingDayValue(workingDayValue) {
    // Array representing the days of the week
    const weekDays = [
      DaysEnum.SAT,
      DaysEnum.FRI,
      DaysEnum.THU,
      DaysEnum.WED,
      DaysEnum.TUE,
      DaysEnum.MON,
      DaysEnum.SUN,
    ];

    // Convert the integer to a binary string and pad with leading zeros to ensure 7 bits
    const binaryString = workingDayValue
      .toString(2)
      .padStart(7, EnableDisableStatusEnum.DISABLED);

    // Initialize an array to hold the days of the week
    const days = [];

    // Iterate through the binary string and weekDays array
    for (let i = 0; i < binaryString.length; i++) {
      if (binaryString[i] === EnableDisableStatusEnum.ENABLED) {
        days.push(weekDays[i]);
      }
    }

    return days;
  }
  timeToMinutes(timeStr) {
    try {
      // Special case for "24:00"
      if (timeStr === CommonHours.TWENTY_FOUR) {
        return CommonHourMinutes.TWENTY_FOUR;
      }

      // Regular expression to validate the 24-hour time format (HH:MM)
      const timePattern = /^([01]\d|2[0-3]):([0-5]\d)$/;
      const match = timeStr.match(timePattern);

      if (!match) {
        throw new Error('Invalid time format');
      }

      // Extract hours and minutes from the matched groups
      const hours = parseInt(match[1], 10);
      const minutes = parseInt(match[2], 10);

      // Calculate the total minutes
      const totalMinutes = hours * 60 + minutes;

      return totalMinutes;
    } catch (error) {
      throw new HttpException(error.message, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  minutesToTime(totalMinutes) {
    try {
      if (
        typeof totalMinutes !== 'number' ||
        totalMinutes < 0 ||
        totalMinutes > CommonHourMinutes.TWENTY_FOUR
      ) {
        throw new Error('Invalid minutes value');
      }

      if (totalMinutes === CommonHourMinutes.TWENTY_FOUR) {
        return CommonHours.TWENTY_FOUR;
      }

      const hours = Math.floor(totalMinutes / 60);
      const minutes = totalMinutes % 60;

      const formattedHours = String(hours).padStart(
        2,
        EnableDisableStatusEnum.DISABLED,
      );
      const formattedMinutes = String(minutes).padStart(
        2,
        EnableDisableStatusEnum.DISABLED,
      );

      return `${formattedHours}:${formattedMinutes}`;
    } catch (error) {
      throw new HttpException(error.message, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  async getDeviceByDeviceUuid(
    deviceUuid: string,
    withProductDevice: boolean = true,
    projectUuid: string,
  ) {
    try {
      return await this.deviceRepository.findOne({
        where: {
          spaceDevice: {
            spaceName: Not(ORPHAN_SPACE_NAME),
            community: {
              project: {
                uuid: projectUuid,
              },
            },
          },
          uuid: deviceUuid,
          isActive: true,
        },
        ...(withProductDevice && { relations: ['productDevice'] }),
      });
    } catch (error) {
      throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
    }
  }
  async addOnlineTemporaryPasswordOneTimeTuya(
    addDeviceObj: addDeviceObjectInterface,
    doorLockUuid: string,
  ): Promise<createTickInterface> {
    try {
      const path = `/v1.0/devices/${doorLockUuid}/door-lock/temp-password`;
      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          name: addDeviceObj.passwordName,
          password: addDeviceObj.encryptedPassword,
          effective_time: addDeviceObj.effectiveTime,
          invalid_time: addDeviceObj.invalidTime,
          password_type: 'ticket',
          ticket_id: addDeviceObj.ticketId,
          type: EnableDisableStatusEnum.ENABLED,
        },
      });

      return response as createTickInterface;
    } catch (error) {
      throw new HttpException(
        error.msg || 'Error adding online temporary password from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  private async validateProject(uuid: string) {
    const project = await this.projectRepository.findOne({
      where: { uuid },
    });
    if (!project) {
      throw new HttpException(
        `A project with the uuid '${uuid}' doesn't exists.`,
        HttpStatus.BAD_REQUEST,
      );
    }
    return project;
  }
}
